# Using serial COM ports

<!-- Table of contents -->
<details>
  <summary markdown="span">
    <b>Contents</b>
  </summary>

  [[_TOC_]]
</details>

**Date:** 2024-07-24

## General notes <a name="chapter__en_general_notes"></a>

The information in this document is provided as is and no guarantee or warranty
is given that the information is fit for any particular purpose. You may not use,
download, upload, copy, print, display, perform, reproduce, publish, license, post,
transmit, or distribute any information from this document in whole or in part
without the express written authorization of ELCOM, spoločnosť s ručením obmedzeným,
Prešov (“COMPANY”). COMPANY reserves the right to change this document at any time,
at its sole discretion, without previous notice.\
COMPANY reserves the right to seek all remedies available by law and in equity
for any violation of these Terms. Any rights not expressly granted herein
are reserved. COMPANY reserves the right to release information to the proper authorities,
as a result of a violation of these Terms, or unlawful acts, if the information
is subpoenaed and/or if we deem it necessary and/or appropriate. COMPANY hereby
disclaims all warranties and conditions with regard to this document, including
all implied warranties and conditions of fitness for a particular purpose, title,
and non-infringement. The information published in this document could include
inaccuracies, errors, or omissions, that be technical or not. Changes are periodically
added to the document. COMPANY may, but is not obligated to, make improvements
and/or changes in the content described herein at any time. Further to what precedes,
COMPANY, its managers and its employees shall in no event be held liable for any
special, direct, indirect, or consequential damages or any damages whatsoever resulting
from technical problems incurred by an equipment such as a breakdown, loss of use, data,
or profits, whether in an action of equity, contract, negligence, or other tortuous
action, arising out of or in connection with the use of this document.
Likewise, COMPANY shall in no event be held liable in case of force majeure.

## About the Document <a name="chapter__en_about_doc"></a>

### History <a name="chapter__en_history"></a>

| Date       | Description                                   |
|------------|-----------------------------------------------|
| 2024-07-24 | Initial version                               |
|            |                                               |
|            |                                               |

## Introduction <a name="chapter__en_introduction"></a>

- Document describes using of serial `COM` ports intended for communication with
  externally connectable devices like:
    - scanners
    - scales
    - payment terminals
    - etc.
- Such `COM` ports can be divided into:
    - Statically created
        - Their kernel devices are always created during Android startup procedure.
        - Such ports are usually implemented using internal `USB` serial converters
          and their outputs are connected to the serial port connector on product body.
    - Dynamically created
        - Their kernel devices are created dynamically when user attach external
          `USB` to serial converter.
        - Supported `USB` to serial converters
            - converters from `FTDI` company

> :warning: **!!! WARNING !!!**
>   - Another prerequisite for creating kernel `COM` port devices (statically
>     and dynamically created) is that product is not connected into PC
>     throught `USB` cable

> :memo: **Notes:**
>   - Document does not describe using of serial `COM` ports used for internal
>     communication between CPU and other internal devices. Such devices
>     are not directly intended to use by developers.

## Identification of assigned kernel device <a name="chapter__en_identify_kernel_dev"></a>

- Kernel device string is neccessary to know for working with serial ports in java applications.
- Serial ports intended for general use by developers are mapped into following
  kernel device strings:

    ```
    /dev/ttyUSB0
    /dev/ttyUSB1
    ...
    /dev/ttyUSB9
    ```
- Mapping physical ports to their kernel devices may not always be the same
  due to existence of dynamically created devices, which are created after
  attaching `USB` to serial converters.

- `SRCH_KERNEL_DEV` table helps to find kernel devices assigned to individual product ports

    - **E100A II product**

        | Serial port identification                | String for searching kernel device |
        |-------------------------------------------|-----------------------------------:|
        | RS232                                     |                          1-1.5:1.0 |
        | USB/UART converter attached on port USB A |                          1-1.4:1.0 |
        | USB/UART converter attached on port USB B |                          1-1.3:1.0 |

- Shell command for search kernel device assigned to port
    ```bash
    ls -la /sys/class/tty/ | grep -oe "<srch_string>/ttyUSB[0-9]" | grep -oe "ttyUSB[0-9]"
    ```

    > :memo: **Notes:**
    >   - `<srch_string>` should be replaced by value from 2nd column of `SRCH_KERNEL_DEV` table
    >   - Answer (for example):
    >       ```
    >       ttyUSB0
    >       ```
    >   - Shell command can be invoked from java code
    >     (see: [Example 1 - Identify kernel device in Java language](#chapter__en_example_1_identify_kernel_dev_java)
    >     file `ExecTerminal.java`)

- Example for searching kernel device assigned to physical port
  (see: [Example 1 - Identify kernel device in Java language](#chapter__en_example_1_identify_kernel_dev_java))

## Working with serial port kernel device <a name="chapter__en_work_with_serial_port_kernel_dev"></a>

- We suggest to use `libserialport` native library for using serial `COM` ports in your Java applications
    - project `libserialport` can be found [https://github.com/sigrokproject/libserialport](https://github.com/sigrokproject/libserialport)
    - prebuilt `.so` library files can be found [https://github.com/Rairmmd/Android-Serial-Port/tree/master/library/libs](https://github.com/Rairmmd/Android-Serial-Port/tree/master/library/libs)
- All next examples uses above mentioned `libserialport` native library

### Adding native libraries into your Android Studio project <a name="chapter__en_add_native_libs_android_studio_prj"></a>

- Follow steps:
    1. Create folder `jniLibs` inside `src/main/`
    2. Put all your `.so` libraries inside `src/main/jniLibs` folder
    3. Folder structure looks like
        ```
        └── app
            └── src
                └── main
                    └── jniLibs
                        ├── armeabi
                        |   └── *.so
                        ├── armeabi-v7a
                        |   └── *.so
                        ├── arm64-v8a
                        |   └── *.so
                        └── x86
                            └── *.so
        ```
    4. No extra code requires just sync your project and run your application

### Loading native library during runtime <a name="chapter__en_load_native_lib_runtime"></a>

- Append following lines into `Java` class which will invokes native `open` and `close` methods
    ```java
    public class SerialPort {
        ...

        // JNI
        private native static FileDescriptor open(String path, int baudrate, int flags);
        public native void close();
        static {
            System.loadLibrary("serial_port");
        }

        ...
    }
    ```

    > :warning: **!!! WARNING !!!**
    > - `Java` class which invokes native `open` and `close` methods must have package
    >   name `android.serialport.api` for correct loading native library:
    >   ```java
    >   package android.serialport.api;
    >   ```

### Openning and closing serial port kernel device <a name="chapter__en_open_close_kernel_dev"></a>

- Use native `open` and `close` methods for opening and closing serial port devices
    > :bell: **Example:**
    >   ```java
    >   private FileDescriptor mFd;
    >
    >   mFd = open( "/dev/ttyUSB0", 115200, 0 );
    >   ...
    >   close();
    >   ```
- Example for `open` and `close` operations
  (see: [Example 2 - Work with serial port kernel device in Java language](#chapter__en_example_2_work_with_kernel_dev_java))

### Writing data to serial port kernel device <a name="chapter__en_write_data_to_kernel_dev"></a>

- Use method `write` of `FileOutputStream` class for writing data to serial port
    > :bell: **Example:**
    >   ```java
    >   private FileOutputStream mFileOutputStream;
    >
    >   mFd = open( "/dev/ttyUSB0", 115200, 0 );
    >   mFileOutputStream = new FileOutputStream( mFd );
    >   mFileOutputStream.write( new String("Hello world!").getBytes() );
    >   ```
- Example for `write` operation
  (see: [Example 2 - Work with serial port kernel device in Java language](#chapter__en_example_2_work_with_kernel_dev_java))

### Reading data from serial port kernel device <a name="chapter__en_read_data_from_kernel_dev"></a>

- Use method `read` of `FileInputStream` class for reading data from serial port
    > :bell: **Example:**
    >   ```java
    >   private FileInputStream mFileInputStream;
    >
    >   mFd = open( "/dev/ttyUSB0", 115200, 0 );
    >   mFileInputStream = new FileInputStream( mFd );
    >
    >   byte[] buffer = new byte[64];
    >   int size = mInputStream.read( buffer );
    >   if (size > 0) {
    >       onDataReceived(buffer, size);
    >   }
    >   ```
- Using of `Thread` class for reading data is highly recommended

## Examples <a name="chapter__en_examples"></a>

### Example 1 - Identify kernel device in Java language <a name="chapter__en_example_1_identify_kernel_dev_java"></a>

- Example describes how to search kernel device for all ports on `E100A II` product.
    - Content of file `ExecTerminal.java`:
        ```java
        package com.example.settingstest;

        import java.io.BufferedReader;
        import java.io.DataInputStream;
        import java.io.DataOutputStream;
        import java.io.IOException;
        import java.io.InputStreamReader;

        import android.util.Log;

        public class ExecTerminal {
            final String TAG =  this.getClass().getName();

            public String exec(String cmd) {
                Log.d(TAG, "^ Executing '" + cmd + "'");
                try {
                    Process process = Runtime.getRuntime().exec("sh");
                    DataInputStream is = new DataInputStream(process.getInputStream());
                    DataOutputStream os = new DataOutputStream(process.getOutputStream());
                    os.writeBytes(cmd + "\n");
                    os.writeBytes("exit\n");
                    os.flush();
                    os.close();

                    BufferedReader reader = new BufferedReader(
                            new InputStreamReader(is));
                    try {
                        String fullOutput = "";
                        String line;
                        while ((line = reader.readLine()) != null) {
                            fullOutput = fullOutput + line + "\n";
                        }
                        return fullOutput;
                    } catch (IOException e) {
                        Log.e(TAG, "^ exec, IOException 1");
                        e.printStackTrace();
                    }

                    process.waitFor();

                } catch (IOException e) {
                    Log.e(TAG, "^ exec, IOException 2");
                    e.printStackTrace();

                } catch (InterruptedException e) {
                    Log.e(TAG, "^ exec, InterruptedException");
                    e.printStackTrace();
                }
                return "";
            }
        }

        ```
    - Content of file `PortFinder.java`:

        ```java
        package com.example.settingstest;

        public class PortFinder {
            enum EProductId {
                E100A_II,
                E70A
            };

            enum EPortId
            {
                E100A_INTERNAL_UART,
                E100A_USB2SERIAL_ON_USB_A,
                E100A_USB2SERIAL_ON_USB_B
            };

            public static String getKernelDev( EProductId product, EPortId port ) {
                String formStr = null;
                String devStr = null;
                String cmdStr = null;
                String kernelPortStr1 = null;
                String kernelPortStr2 = "ttyUSB[0-9]";

                if( product == EProductId.E100A_II) {
                    switch( port )
                    {
                        case E100A_INTERNAL_UART:
                            kernelPortStr1 = "1-1.5:1.0";
                            break;
                        case E100A_USB2SERIAL_ON_USB_A:
                            kernelPortStr1 = "1-1.4:1.0";
                            break;
                        case E100A_USB2SERIAL_ON_USB_B:
                            kernelPortStr1 = "1-1.3:1.0";
                            break;
                    }
                    if( kernelPortStr1 != null ) {
                        formStr = "ls -la /sys/class/tty/ | grep -oe \"%s/%s\" | grep -oe \"%s\"";
                        cmdStr = String.format( formStr, kernelPortStr1, kernelPortStr2, kernelPortStr2 );
                    }
                }
                else {
                    //product E70A is not supported yet
                }

                if( cmdStr != null )
                {
                    ExecTerminal terminal = new ExecTerminal();
                    String cmdOutput = terminal.exec( cmdStr );
                    cmdOutput = cmdOutput.replaceAll("[\r\n]", "" );
                    if( cmdOutput.equals( "" ) ){
                        devStr = null;
                    }
                    else{
                        devStr = String.format("/dev/%s", cmdOutput);
                    }
                }
                return devStr;
            }
        }
        ```
    - Content of file `MainActivity.java`:
        ```java
        package com.example.settingstest;

        import androidx.appcompat.app.AppCompatActivity;

        import android.content.ActivityNotFoundException;
        import android.content.Intent;
        import android.os.Bundle;
        import android.util.Log;
        import android.view.View;
        import android.widget.AdapterView;
        import android.widget.ArrayAdapter;
        import android.widget.Button;
        import android.widget.Spinner;
        import android.widget.TextView;

        public class MainActivity extends AppCompatActivity {
            @Override
            protected void onCreate(Bundle savedInstanceState) {
                super.onCreate(savedInstanceState);
                setContentView(R.layout.activity_main);

                Button button1 = (Button) findViewById(R.id.button1);
                button1.setOnClickListener(new View.OnClickListener() {
                    public void onClick(View v) {
                        String resultStr = "";
                        String kernelDev = null;

                        kernelDev = PortFinder.getKernelDev( PortFinder.EProductId.E100A_II,
                                                            PortFinder.EPortId.E100A_INTERNAL_UART );
                        resultStr += "internal uart -> ";
                        if( kernelDev != null ) {
                            resultStr += kernelDev + "\n";
                        }
                        else{
                            resultStr += "not connected" + "\n";
                        }

                        kernelDev = PortFinder.getKernelDev( PortFinder.EProductId.E100A_II,
                                PortFinder.EPortId.E100A_USB2SERIAL_ON_USB_A );
                        resultStr += "port USB A -> ";
                        if( kernelDev != null ) {
                            resultStr += kernelDev + "\n";
                        }
                        else{
                            resultStr += "not connected" + "\n";
                        }

                        kernelDev = PortFinder.getKernelDev( PortFinder.EProductId.E100A_II,
                                PortFinder.EPortId.E100A_USB2SERIAL_ON_USB_B );
                        resultStr += "port USB B -> ";
                        if( kernelDev != null ) {
                            resultStr += kernelDev + "\n";
                        }
                        else{
                            resultStr += "not connected" + "\n";
                        }

                        TextView myView = (TextView) findViewById(R.id.textView);
                        myView.setText( resultStr );
                    }
                });
            }
        }
        ```

### Example 2 - Work with serial port kernel device in Java language <a name="chapter__en_example_2_work_with_kernel_dev_java"></a>

- Example describes how to open, write data and close serial port kernel device
    - Content of file `SerialPort.java`:
        ```java
        package android.serialport.api;

        import java.io.File;
        import java.io.FileDescriptor;
        import java.io.FileInputStream;
        import java.io.FileOutputStream;
        import java.io.IOException;
        import java.io.InputStream;
        import java.io.OutputStream;

        import android.util.Log;

        public class SerialPort {

            private static final String TAG = "SerialPort";

            private FileDescriptor mFd;
            private FileInputStream mFileInputStream;
            private FileOutputStream mFileOutputStream;

            public SerialPort(String path, int baudrate, int flags) throws IOException {
                mFd = open( path, baudrate, flags);
                if (mFd == null) {
                    Log.e(TAG, "native open returns null");
                    throw new IOException();
                }
                mFileInputStream = new FileInputStream(mFd);
                mFileOutputStream = new FileOutputStream(mFd);
            }

            public InputStream getInputStream() {
                return mFileInputStream;
            }
            public OutputStream getOutputStream() {
                return mFileOutputStream;
            }

            // JNI
            private native static FileDescriptor open(String path, int baudrate, int flags);
            public native void close();
            static {
                System.loadLibrary("serial_port");
            }
        }

        ```
    - Content of file `MainActivity.java`:
        ```java
        package com.example.settingstest;

        import androidx.appcompat.app.AppCompatActivity;

        import android.app.AlertDialog;
        import android.os.Bundle;
        import android.view.View;
        import android.widget.Button;
        import android.widget.EditText;
        import android.widget.TextView;

        import java.io.IOException;

        import android.serialport.api.SerialPort;

        public class MainActivity extends AppCompatActivity {
            @Override
            protected void onCreate(Bundle savedInstanceState) {
                super.onCreate(savedInstanceState);
                setContentView(R.layout.activity_main);

                Button button2 = (Button) findViewById(R.id.button);
                button2.setOnClickListener(new View.OnClickListener() {
                    public void onClick(View v) {
                        try {
                            SerialPort port = new SerialPort( "/dev/ttyUSB0", 115200, 0 );
                            EditText myEditText = (EditText) findViewById(R.id.editTextTextMultiLine);
                            port.getOutputStream().write( myEditText.getText().toString().getBytes() );
                            port.close();
                        } catch (IOException e) {
                            DisplayError("Opening port error");
                        }
                    }
                });
            }

            private void DisplayError(String message) {
                AlertDialog.Builder b = new AlertDialog.Builder(this);
                b.setTitle("Error");
                b.setMessage(message);
                b.setPositiveButton("OK",null);
                b.show();
            }
        }
        ```
